/*
  SCBus firmware example using a pseudo-random number
  generator to draw dice.  This example uses a fixed
  seed, so the sequence of "rolls" will repeat each
  time the program is restarted.
  
  Copyright (c) 2014 MetaMorph, Inc.
  
 */

int seed = 123;

const char * dTop   = ".---------.";
const char * dBlank = "|         |";
const char * dRight = "|       @ |";
const char * dLeft  = "| @       |";
const char * dMiddle= "|    @    |";
const char * dBoth  = "| @     @ |";
const char * dBottom= "'---------'";

const char * dStrings[] =
{ dTop, dBlank, dRight, dLeft, dMiddle, dBoth, dBottom };

#define DICELIST_SZ (7)

typedef byte diceList_t[DICELIST_SZ];

diceList_t dicePattern[] =
{
  {0, 1, 1, 1, 1, 1, 6},  // dummy "0" die
  {0, 1, 1, 4, 1, 1, 6},  // "1" die
  {0, 2, 1, 1, 1, 3, 6},  // "2" die
  {0, 2, 1, 4, 1, 3, 6},  // "3" die
  {0, 5, 1, 1, 1, 5, 6},  // "4" die
  {0, 5, 1, 4, 1, 5, 6},  // "5" die
  {0, 5, 1, 5, 1, 5, 6}   // "6" die
};


#define NUMBER_OF_DICE (2)
byte dice[ NUMBER_OF_DICE ];

void setup() {
  Serial.begin(9600);
  randomSeed( seed );
}

void loop() {
  if (Serial.available() > 0) {
    int inByte = Serial.read();
    
    // Send back some data
    Serial.write(inByte + 1);
    Serial.write("\n");
    
    // Compute some dice
    for( int i = 0; i < NUMBER_OF_DICE; i++ )
    {
      dice[ i ] = random( 1, 6+1 );
    }
    // Print the dice
    for( int line = 0; line < DICELIST_SZ; line++ )
    {
      int patternIndex = dicePattern[ dice[ 0 ] ][ line ];
      Serial.write( dStrings[ patternIndex ] );
      Serial.write( ' ' );
      patternIndex = dicePattern[ dice[ 1 ] ][ line ];
      Serial.write( dStrings[ patternIndex ] );
      Serial.write( '\n' );
    }
  }
}
